	function J = objfun(theta)
%
%	OBJFUN E-FRIT̕]֐
%
%	Output:
%		J		: objective function
%	Input:
%		theta	: vector of optimization variables
%		
%	--- update history ---
%	original  (02/10,2009): K. Tasaka and M. Kano
%	updated 0 (03/31,2010): M. Ogawa
%		>> use NLP instead of PSO for faster optimization.
%	updated 1 (05/22,2010): M. Ogawa and M. Kano
%		>> minor modification for the release
%	updated 2 (05/27,2010): M. Ogawa and M. Kano
%		>> add PID
%
%	Copyright (C) 2009-2010 Kenichi Tasaka, Manabu Kano, Morimasa Ogawa.
%	All rights reserved. 
%	{vOі{vOύXvO̍Ĕzzւ܂D
%
	global  y0  u0  r0  t0
	global  PID_algorithm  dir_rev  gamma
	global  n  Tn
	global  fs  lambda
	global  rt  yt
%
% --------------
% --- set up ---
% --------------
%
	kmax = length(t0);
	rt   = zeros(kmax,1);
	yt   = zeros(kmax,1);
	ut   = zeros(kmax,1);
	e    = zeros(kmax,1);
	du   = zeros(kmax,1);

	Kc = theta(1);
	Ti = theta(2);
	Td = theta(3);
	TL = theta(4);

	if Td < 0		% to improve robustness of fmincon
		Td = 0;
	end;
%
% --------------------------
% --- PID control system ---
% --------------------------
%
% --- C and C inverse ---
%
	num_C = dir_rev * Kc * [ ( 1 + ( 1 / gamma ) ) * Ti * Td  ( Ti + ( Td / gamma ) )  1 ];
	den_C = [  ( Ti * Td / gamma )  Ti  0 ];
	C     = tf( num_C,den_C );
	Cinv  = tf( den_C,num_C );
%
% --- desired response ---
%
	M = reference_model(Tn,TL,n);
%
% --- set-point filter ---
%
	switch PID_algorithm;
	case 1; % PID algorithm
		num_F = [ 1 ];
		den_F = [ 1 ];
		F = tf( num_F,den_F );
	case 2; % PI-D algorithm
		num_F = [ Ti * Td / gamma   ( Ti + Td / gamma )   1 ];
		den_F = [ ( 1 + ( 1 / gamma ) ) * Ti * Td   ( Ti + Td / gamma )   1 ];
		F = tf( num_F,den_F );
	case 3; % I-PD algorithm
		num_F = [ Td / gamma   1 ];
		den_F = [ ( 1 + ( 1 / gamma ) ) * Ti * Td   ( Ti + Td / gamma )   1 ];
		F = tf( num_F,den_F );
	otherwise;
		disp('unknown PID algorithm');
	end;
%
% --------------------------
% --- objective function ---
% --------------------------
%
% --- ( y0 - yt ) --
%
	rt = lsim(Cinv,u0,t0) + y0;
	yt = lsim( M/F,rt,t0 );
	e  = y0 - yt;
%
% --- du ---
%
	yt0 = lsim( M,r0,t0 );
	rf0 = lsim( F,r0,t0 );
	ut0 = lsim( C,rf0-yt0,t0 );
	du  = ut0(1:kmax) - [ 0; ut0(1:kmax-1) ];
%
% --- quadratic objective function ---
%
	Jy = ( e' * e ) / length(e);
	Ju = ( lambda * fs )^2 * ( du' * du ) / length(du);
	J  = Jy + Ju;
	return;
